<?php
/**
 * This plugin utilizes Open Source code. Details of these open source projects along with their licenses can be found below.
 * We acknowledge and are grateful to these developers for their contributions to open source.
 *
 * Project: mobile-dj-manager https://github.com/deckbooks/mobile-dj-manager
 * License: (GNU General Public License v2.0) https://github.com/deckbooks/mobile-dj-manager/blob/master/license.txt
 *
 * @author: Mike Howard, Jack Mawhinney, Dan Porter
 *
 * Admin Notices
 *
 * @package     TMEM
 * @subpackage  Admin/Notices
 * @copyright   Copyright (c) 2016, Mike Howard
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       1.3
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Retrieve all dismissed notices.
 *
 * @since   1.5
 * @return  array   Array of dismissed notices
 */
function tmem_dismissed_notices() {

	global $current_user;

	$user_notices = (array) get_user_option( 'tmem_dismissed_notices', $current_user->ID );

	return $user_notices;

} // tmem_dismissed_notices

/**
 * Check if a specific notice has been dismissed.
 *
 * @since   1.5
 * @param   string $notice Notice to check.
 * @return  bool    Whether or not the notice has been dismissed
 */
function tmem_is_notice_dismissed( $notice ) {

	$dismissed = tmem_dismissed_notices();

	if ( array_key_exists( $notice, $dismissed ) ) {
		return true;
	} else {
		return false;
	}

} // tmem_is_notice_dismissed

/**
 * Dismiss a notice.
 *
 * @since   1.5
 * @param   string $notice Notice to dismiss.
 * @return  bool|int    True on success, false on failure, meta ID if it didn't exist yet
 */
function tmem_dismiss_notice( $notice ) {

	global $current_user;

	$new = (array) tmem_dismissed_notices();
	$dismissed_notices = $new;

	if ( ! array_key_exists( $notice, $dismissed_notices ) ) {
		$new[ $notice ] = 'true';
	}

	$update = update_user_option( $current_user->ID, 'tmem_dismissed_notices', $new );

	return $update;

} // tmem_dismiss_notice

/**
 * Restore a dismissed notice.
 *
 * @since   1.5
 * @param   string $notice Notice to restore.
 * @return  bool|int    True on success, false on failure, meta ID if it didn't exist yet
 */
function tmem_restore_notice( $notice ) {

	global $current_user;

	$dismissed_notices = (array) tmem_dismissed_notices();

	if ( array_key_exists( $notice, $dismissed_notices ) ) {
		unset( $dismissed_notices[ $notice ] );
	}

	$update = update_user_option( $current_user->ID, 'tmem_dismissed_notices', $dismissed_notices );

	return $update;

} // tmem_restore_notice

/**
 * Admin Messages
 *
 * @since   1.3
 * @global  $tmem_options   Array of all the TMEM Options
 * @return void
 */
function tmem_admin_notices() {
	global $tmem_options;

	// Unattended events.
	if ( tmem_employee_can( 'manage_all_events' ) && ( tmem_get_option( 'warn_unattended' ) ) ) {
		$unattended = tmem_event_count( 'tmem-unattended' );

		if ( ! empty( $unattended ) && $unattended > 0 ) {
			echo '<div class="notice notice-info is-dismissible">';
			echo '<p>' .
					sprintf(
						/* translators: %s is the link to the Unattended Enquiries area. */
						__( 'You have unattended enquiries. <a href="%s">Click here</a> to manage.', 'mobile-events-manager' ), // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
						admin_url( 'edit.php?post_type=tmem-event&post_status=tmem-unattended' ) // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					) . '</p>';
			echo '</div>';
		}
	}

	// Settings.
	if ( isset( $_GET['tmem-message'] ) && 'upgrade-completed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-upgraded',
			__( 'Mobile Events Manager has been upgraded successfully.', 'mobile-events-manager' ),
			'updated'
		);
	}

	// Availability.
	if ( isset( $_GET['tmem-message'] ) && 'absence-added' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-absence-added',
			__( 'Absence added.', 'mobile-events-manager' ),
			'updated'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'absence-fail' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-absence-fail',
			__( 'Absence could not be added.', 'mobile-events-manager' ),
			'error'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'absence-removed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-absence-deleted',
			__( 'Absence deleted.', 'mobile-events-manager' ),
			'updated'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'absence-delete-fail' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-absence-remove-fail',
			__( 'Absence could not be deleted.', 'mobile-events-manager' ),
			'error'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'song_added' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-added-song',
			__( 'Entry added to playlist.', 'mobile-events-manager' ),
			'updated'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'adding_song_failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-adding-song-failed',
			__( 'Could not add entry to playlist.', 'mobile-events-manager' ),
			'error'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'song_removed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-removed-song',
			__( 'The selected songs were removed.', 'mobile-events-manager' ),
			'updated'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'song_remove_failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-remove-faled',
			__( 'The songs count not be removed.', 'mobile-events-manager' ),
			'error'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'security_failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-security-failed',
			__( 'Security verification failed. Action not completed.', 'mobile-events-manager' ),
			'error'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'playlist_emailed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-playlist-emailed',
			__( 'The event details were emailed successfully.', 'mobile-events-manager' ),
			'updated'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'playlist_email_failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-playlist-email-failed',
			__( 'The event details could not be emailed. Check there is an email address associated to the event staff.', 'mobile-events-manager' ),
			'error'
		);
	}

	if ( isset( $_GET['tmem-message'] ) && 'employee_added' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-employee_added',
			__( 'Employee added.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'employee_add_failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-employee_add-failed',
			__( 'Could not add employee.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'employee_info_missing' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-employee_info-missing',
			__( 'Insufficient information to create employee.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'comm_missing_content' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-comm_content-missing',
			__( 'Not all required fields have been completed.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'comm_sent' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-comm_sent',
			__( 'Email sent successfully.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'comm_not_sent' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-comm_not_sent',
			__( 'Email not sent.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-action'] ) && 'get_event_availability' === $_GET['tmem-action'] ) {

		if ( ! isset( $_GET['tmem_nonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['tmem_nonce'], 'get_event_availability' ) ) ) ) {
			return;
		} elseif ( ! isset( $_GET['event_id'] ) ) {
			return;
		} else {

			$date = get_post_meta( absint( wp_unslash( $_GET['event_id'] ) ), '_tmem_event_date', true );

			$result = tmem_do_availability_check( $date );

			if ( ! empty( $result['available'] ) ) {

				echo '<ul>';

				foreach ( $result['available'] as $employee_id ) {
					echo '<li>';
						printf(
							/* translators: %2 employee %s plural */
							__( '<a href="%1$s" title="Assign &amp; Respond to Enquiry">Assign %2$s &amp; respond to enquiry</a>', 'mobile-events-manager' ), // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
							add_query_arg( // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
								'primary_employee',
								$employee_id,
								get_edit_post_link( absint( wp_unslash( $_GET['event_id'] ) ) )
							),
							tmem_get_employee_display_name( $employee_id ) // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
						);
					echo '</li>';
				}

				echo '</ul>';

				echo '<div class="notice notice-info is-dismissible">';
					echo '<p>';
					printf(
						/* translators: %1: number of employees $s plural %2: contract ID %3: date $d %4 */
						esc_html__( 'You have %1$d employees available to work %2$s %3$s on %4$s.', 'mobile-events-manager' ),
						count( $result['available'] ),
						esc_html( tmem_get_label_singular( true ) ),
						esc_html( tmem_get_event_contract_id( absint( wp_unslash( $_GET['event_id'] ) ) ) ),
						esc_html( tmem_get_event_long_date( absint( wp_unslash( $_GET['event_id'] ) ) ) )
					);
					echo '</p>';
				echo '</div>';

			} else {

				echo '<div class="notice notice-error is-dismissible">';
					echo '<p>';
					printf(
						/* translators: %1 event type %2 contract ID %3 long date $s plural */
						esc_html__( 'There are no employees available to work %1$s %2$s on %3$s', 'mobile-events-manager' ),
						esc_html( tmem_get_label_singular( true ) ),
						esc_html( tmem_get_event_contract_id( absint( wp_unslash( $_GET['event_id'] ) ) ) ),
						esc_html( tmem_get_event_long_date( absint( wp_unslash( $_GET['event_id'] ) ) ) )
					);
					echo '</p>';
				echo '</div>';

			}
		}
	}
	if ( isset( $_GET['tmem-message'] ) && 'payment_event_missing' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-payment_event_missing',
			__( 'Event not identified.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'pay_employee_failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-payment_employee_failed',
			__( 'Unable to make payment to employee.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'pay_all_employees_failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-payment_employees_failed',
			__( 'Unable to make payment to employees.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'pay_all_employees_some_success' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-payment_all_employees_some_success',
			__( 'Not all employees could be paid.', 'mobile-events-manager' ),
			'notice-info'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'pay_employee_success' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-payment_employeee_success',
			__( 'Employee successfully paid.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'pay_all_employees_success' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-payment_all_employeees_success',
			__( 'Employees successfully paid.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'unattended_enquiries_rejected_success' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-unattended_enquiries_rejected_success',
			sprintf(
				/* translators: %1 event type %2 contract id %3 event id $s plural */
				_n( '%1$s %2$s successfully rejected.', '%1$s %3$s successfully rejected.', isset( $_GET['tmem-count'] ) ? absint( wp_unslash( $_GET['tmem-count'] ) ) : 0, 'mobile-events-manager' ),
				isset( $_GET['tmem-count'] ) ? absint( wp_unslash( $_GET['tmem-count'] ) ) : 0,
				esc_html( tmem_get_label_singular() ),
				esc_html( tmem_get_label_plural() )
			),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'unattended_enquiries_rejected_failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-unattended_enquiries_rejected_failed',
			__( 'Errors were encountered.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'api-key-generated' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-api-key-generated',
			__( 'API keys generated.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'api-key-regenerated' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-api-key-regenerated',
			__( 'API keys re-generated.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'api-key-revoked' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-api-key-revoked',
			__( 'API keys revoked.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'api-key-failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-api-key-failed',
			__( 'Generating API keys failed.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'task-status-updated' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-task-status-updated',
			__( 'Task status updated.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'task-status-update-failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-task-status-update-failed',
			__( 'Task status could not be updated.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'task-run' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-task-run',
			__( 'Task executed successfully.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'task-run-failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-run-failed',
			__( 'Task could not be executed.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'task-updated' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-task-updated',
			__( 'Task updated.', 'mobile-events-manager' ),
			'updated'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'task-update-failed' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-update-failed',
			__( 'Task update failed.', 'mobile-events-manager' ),
			'error'
		);
	}
	if ( isset( $_GET['tmem-message'] ) && 'settings-imported' === $_GET['tmem-message'] ) {
		add_settings_error(
			'tmem-notices',
			'tmem-settings-imported',
			__( 'Settings sucessfully imported.', 'mobile-events-manager' ),
			'updated'
		);
	}

	settings_errors( 'tmem-notices' );

} // tmem_admin_messages
add_action( 'admin_notices', 'tmem_admin_notices' );

/**
 * Admin WP Rating Request Notice
 *
 * @since   1.5
 * @return  void
 */
function tmem_admin_wp_5star_rating_notice() {
	ob_start(); ?>

	<div class="updated notice notice-tmem-dismiss is-dismissible" data-notice="tmem_request_wp_5star_rating">
		<p>
			<strong><?php esc_html_e( 'Thank you for using Mobile Events Manager!', 'mobile-events-manager' ); ?></strong>
		</p>
		<p>
			<?php
			printf(
				/* translators: $1 WordPress link  $s plural */
				__( 'Would you <strong>please</strong> do us a favour and leave a 5 star rating on WordPress.org? It only takes a minute and it <strong>really helps</strong> our developers to continue to work on great new features and functionality. <a href="%1$s" target="_blank">Yes, most definitely!</a>', 'mobile-events-manager' ), // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				'https://wordpress.org/support/plugin/mobile-events-manager/reviews/'
			);
			?>
		</p>
			<p>
			<?php
			printf(
				__( 'If you haven&#39;t already, please check out our <a href="https://www.mobileeventsmanager.co.uk/Add-Ons">Add Ons</a> to further enhance Mobile Events Manager.', 'mobile-events-manager' ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			?>
		</p>
			<p>
			<?php
			printf(
				__( 'You can also join our Mobile Events Manager User Group on <a href="https://www.facebook.com/groups/mobileeventsmanager">Facebook</a>. Don&#39;t forget to Like our <a href="https://www.facebook.com/mobileeventsmanager">Facebook Page</a> too.', 'mobile-events-manager' ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped 
			?>
		</p>
	</div>

	<?php
	echo ob_get_clean(); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
} // tmem_admin_wp_5star_rating_notice

/**
 * Request 5 star rating after 1 event has been completed.
 *
 * After 1 completed event we ask the admin for a 5 star rating on WordPress.org
 *
 * @since   1.3
 * @return  void
 */
function tmem_request_wp_5star_rating() {

	global $typenow, $pagenow;

	$allowed_types = array(
		'tmem-event',
		'tmem-package',
		'tmem-addon',
		'tmem_communication',
		'contract',
		'email_template',
		'tmem-playlist',
		'tmem-transaction',
		'tmem-venue',
	);
	$allowed_pages = array( 'edit.php', 'post.php', 'post-new.php', 'index.php', 'plugins.php' );

	if ( ! current_user_can( 'administrator' ) ) {
		return;
	}

	if ( ! in_array( $typenow, $allowed_types ) && ! in_array( $pagenow, $allowed_pages ) ) {
		return;
	}

	if ( tmem_is_notice_dismissed( 'tmem_request_wp_5star_rating' ) ) {
		return;
	}

	global $wpdb;

	$completed_events = $wpdb->get_var(
		$wpdb->prepare(
			"
			SELECT COUNT(*)
			FROM $wpdb->posts
			WHERE `post_type` = %s
			AND `post_status` = %s
		",
			'tmem-event',
			'tmem-approved'
		)
	);

	if ( $completed_events >= 1 ) {
		add_action( 'admin_notices', 'tmem_admin_wp_5star_rating_notice' );
	}

} // tmem_request_wp_5star_rating
add_action( 'plugins_loaded', 'tmem_request_wp_5star_rating' );


/**
 * Implements tmem_admin_wp_addons().
 *
 * Description Invites user to look at additional addons.
 */
function tmem_admin_wp_addons() {
	ob_start();
	?>

	<div class="updated notice notice-tmem-dismiss is-dismissible" data-notice="tmem_request_wp_addons">
		<p>
			<strong><?php esc_html_e( 'Thank you for installing Mobile Events Manager!', 'mobile-events-manager' ); ?></strong>
		</p>
		<p>
			<?php
			printf(
				__( 'Did you know we have a few addons to further enhance your Mobile Events Manager Plugin? <a href="https://www.mobileeventsmanager.co.uk/downloads/stripe-payments/">Stripe Payments</a> and <a href="https://www.mobileeventsmanager.co.uk/downloads/enquiry-forms/">Enquiry Forms</a>.', 'mobile-events-manager' ), // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				''
			); // invite to download new addons.
			?>
		</p>
			<p>
			<?php
			printf(
				__( 'You can also join our Mobile Events Manager User Group on <a href="https://www.facebook.com/groups/mobileeventsmanager">Facebook</a>. Don&#39;t forget to Like our <a href="https://www.facebook.com/mobileeventsmanager">Facebook Page</a> too.', 'mobile-events-manager' ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			?>
		</p>
	</div>

	<?php
	echo ob_get_clean(); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
} // tmem_admin_wp_addons

/**
 * Link to visit Addons pages.
 *
 * @since   1.3
 * @return  void
 */
function tmem_request_wp_addons() {

	global $typenow, $pagenow;

	$allowed_types = array(
		'tmem-event',
		'tmem-package',
		'tmem-addon',
		'tmem_communication',
		'contract',
		'email_template',
		'tmem-playlist',
		'tmem-transaction',
		'tmem-venue',
	);
	$allowed_pages = array( 'edit.php', 'post.php', 'post-new.php', 'index.php', 'plugins.php' );

	if ( ! current_user_can( 'administrator' ) ) {
		return;
	}

	if ( ! in_array( $typenow, $allowed_types ) && ! in_array( $pagenow, $allowed_pages ) ) {
		return;
	}

	if ( tmem_is_notice_dismissed( 'tmem_request_wp_addons' ) ) {
		return;
	}

	global $wpdb;

	$addon_events = $wpdb->get_var(
		$wpdb->prepare(
			"
			SELECT COUNT(*)
			FROM $wpdb->posts
			WHERE `post_type` = %s
			AND `post_status` = %s
		",
			'tmem-event',
			'tmem-enquiry'
		)
	);

	if ( $addon_events >= 0 ) {
		add_action( 'admin_notices', 'tmem_admin_wp_addons' );
	}

} // tmem_request_wp_addons
add_action( 'plugins_loaded', 'tmem_request_wp_addons' );
