<?php
/**
 * This plugin utilizes Open Source code. Details of these open source projects along with their licenses can be found below.
 * We acknowledge and are grateful to these developers for their contributions to open source.
 *
 * Project: mobile-dj-manager https://github.com/deckbooks/mobile-dj-manager
 * License: (GNU General Public License v2.0) https://github.com/deckbooks/mobile-dj-manager/blob/master/license.txt
 *
 * @author: Mike Howard, Jack Mawhinney, Dan Porter
 *
 * Weclome Page Class
 *
 * @package     TMEM
 * @subpackage  Admin/Welcome
 * @copyright   Copyright (c) 2016, Mike Howard
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       1.3
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * TMEM_Welcome Class
 *
 * A general class for About and Credits page.
 *
 * @since   1.3
 */
class TMEM_Welcome {

	/**
	 * @var string The capability users should have to view the page
	 */
	public $minimum_capability = 'manage_options';

	/**
	 * Get things started
	 *
	 * @since   1.3
	 */
	public function __construct() {
		add_action( 'admin_menu', array( $this, 'admin_menus' ) );
		add_action( 'admin_head', array( $this, 'admin_head' ) );
		add_action( 'admin_init', array( $this, 'welcome' ) );
	}

	/**
	 * Register the Dashboard Pages which are later hidden but these pages
	 * are used to render the Welcome and Credits pages.
	 *
	 * @access public
	 * @since 1.3
	 * @return void
	 */
	public function admin_menus() {

		// Changelog Page
		add_dashboard_page(
			__( 'Mobile Events Manager Changelog', 'mobile-events-manager' ),
			__( 'Mobile Events Manager Changelog', 'mobile-events-manager' ),
			$this->minimum_capability,
			'tmem-changelog',
			array( $this, 'changelog_screen' )
		);

		// Getting Started Page
		add_dashboard_page(
			__( 'Getting started with Mobile Events Manager', 'mobile-events-manager' ),
			__( 'Getting started with Mobile Events Manager', 'mobile-events-manager' ),
			$this->minimum_capability,
			'tmem-getting-started',
			array( $this, 'getting_started_screen' )
		);

		// Now remove them from the menus so plugins that allow customizing the admin menu don't show them
		remove_submenu_page( 'index.php', 'tmem-changelog' );
		remove_submenu_page( 'index.php', 'tmem-getting-started' );

	} // admin_menus

	/**
	 * Hide Individual Dashboard Pages
	 *
	 * @access  public
	 * @since   1.3
	 * @return  void
	 */
	public function admin_head() {
		?>
		<style type="text/css" media="screen">
			/*<![CDATA[*/
			.tmem-about-wrap .tmem-badge { float: right; border-radius: 4px; margin: 0 0 15px 15px; max-width: 200px; }
			.tmem-about-wrap #tmem-header { margin-bottom: 15px; }
			.tmem-about-wrap #tmem-header h1 { margin-bottom: 15px !important; }
			.tmem-about-wrap .about-text { margin: 0 0 15px; max-width: 670px; }
			.tmem-about-wrap .feature-section { margin-top: 20px;  }
			.tmem-about-wrap .feature-section-content,
			.tmem-about-wrap .feature-section-media { width: 50%; box-sizing: border-box; }
			.tmem-about-wrap .feature-section-content { float: left; padding-right: 50px; }
			.tmem-about-wrap .feature-section-content h4 { margin: 0 0 1em; }
			.tmem-about-wrap .feature-section-media { float: right; text-align: right; margin-bottom: 20px; }
			.tmem-about-wrap .feature-section-media img { border: 1px solid #ddd; }
			.tmem-about-wrap .feature-section:not(.under-the-hood) .col { margin-top: 0; }
			.tmem-about-wrap .changelog, .tmem-about-wrap .return-to-dashboard { clear: both; }
			/* responsive */
			@media all and ( max-width: 782px ) {
				.tmem-about-wrap .feature-section-content,
				.tmem-about-wrap .feature-section-media { float: none; padding-right: 0; width: 100%; text-align: left; }
				.tmem-about-wrap .feature-section-media img { float: none; margin: 0 0 20px; }
			}
			/*]]>*/
		</style>
		<?php
	} // admin_head

	/**
	 * Welcome message
	 *
	 * @access public
	 * @since 2.5
	 * @return void
	 */
	public function welcome_message() {
		list( $display_version ) = explode( '-', TMEM_VERSION_NUM );
		?>
		<div id="tmem-header">
			<img class="tmem-badge" src="<?php echo esc_url( TMEM_PLUGIN_URL ) . '/assets/images/tmem_logo_300.png'; ?>" alt="<?php esc_attr_e( 'Mobile Events Manager', 'mobile-events-manager' ); ?>" / >
			<h1><?php printf( esc_html__( 'Welcome to Mobile Events Manager %s', 'mobile-events-manager' ), esc_html( $display_version ) ); ?></h1>
			<p class="about-text">
				<?php esc_html_e( 'Thank you for choosing our plugin to run your event\'s business!', 'mobile-events-manager' ); ?>
				<br />
				<?php
					printf(
						esc_html__( 'Mobile Events Manager %1$s is ready to make your %2$s business even more efficient!', 'mobile-events-manager' ),
						esc_html( $display_version ),
						esc_html( tmem_get_label_plural( true ) )
					);
				?>
			</p>
		</div>
		<?php
	} // welcome_message

	/**
	 * Navigation tabs
	 *
	 * @access  public
	 * @since   1.3
	 * @return  void
	 */
	public function tabs() {
		$selected = isset( $_GET['page'] ) ? sanitize_text_field( wp_unslash( $_GET['page'] ) ) : 'tmem-about';
		?>
		<h1 class="nav-tab-wrapper">
			<a class="nav-tab <?php echo $selected == 'tmem-getting-started' ? 'nav-tab-active' : ''; ?>" href="<?php echo esc_url( admin_url( add_query_arg( array( 'page' => 'tmem-getting-started' ), 'index.php' ) ) ); ?>">
				<?php esc_html_e( 'Let\'s Get Started', 'mobile-events-manager' ); ?>
			</a>
		</h1>
		<?php
	} // tabs

	/**
	 * Render Changelog Screen
	 *
	 * @access public
	 * @since 2.0.3
	 * @return void
	 */
	public function changelog_screen() {
		?>
		<div class="wrap about-wrap tmem-about-wrap">
			<?php
				// load welcome message and content tabs
				$this->welcome_message();
				$this->tabs();
			?>
			<div class="changelog">
				<h3><?php esc_html_e( 'Full Mobile Events Manager Changelog', 'mobile-events-manager' ); ?></h3>

				<div class="feature-section">
					<?php echo $this->parse_readme(); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
				</div>
			</div>

			<div class="return-to-dashboard">
				<a href="
				<?php
				echo esc_url(
					admin_url(
						add_query_arg(
							array(
								'post_type' => 'tmem-event',
								'page'      => 'tmem-settings',
							),
							'edit.php'
						)
					)
				);
				?>
							"><?php esc_html_e( 'Go to Mobile Events Manager Settings', 'mobile-events-manager' ); ?></a>
			</div>
		</div>
		<?php
	} // changelog_screen

	/**
	 * Render Getting Started Screen
	 *
	 * @access public
	 * @since 1.9
	 * @return void
	 */
	public function getting_started_screen() {
		?>
		<div class="wrap about-wrap tmem-about-wrap">
			<?php
				// load welcome message and content tabs
				$this->welcome_message();
				$this->tabs();
			?>
			<p class="about-description"><?php esc_html_e( "Now that Mobile Events Manager is installed, you're ready to get started. It works out of the box, but there are some customisations you can configure to match your business needs.", 'mobile-events-manager' ); ?></p>

			<div class="changelog">
				<h3><?php printf( esc_html__( 'Creating Your First %s', 'mobile-events-manager' ), esc_html( tmem_get_label_singular() ) ); ?></h3>
				<div class="feature-section">
					<div class="feature-section-media">
						<img src="<?php echo esc_url( TMEM_PLUGIN_URL ) . '/assets/images/screenshots/mem_create_event.jpg'; ?>" class="tmem-welcome-screenshots"/>
					</div>
					<div class="feature-section-content">
						<h4><a href="<?php echo esc_url( admin_url( 'post-new.php?post_type=tmem-event' ) ); ?>"><?php printf( esc_html__( 'TMEM %1$s &rarr; Create %2$s', 'mobile-events-manager' ), esc_html( tmem_get_label_plural() ), esc_html( tmem_get_label_singular() ) ); ?></a></h4>
						<p><?php printf( __( 'The TMEM %1$s menu is your access point to all aspects of your %2$s creation and setup. To create your first %2$s, simply click Add New and then fill out the %2$s details.', 'mobile-events-manager' ), tmem_get_label_plural(), tmem_get_label_singular( true ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>

						<h4><?php esc_html_e( 'Create a Client', 'mobile-events-manager' ); ?></h4>
						<p><?php printf( __( 'Create clients directly from the %s screen by selected <strong>Add New Client</strong> from the <em>Select Client</em> dropdown to reveal a few additional fields', 'mobile-events-manager' ), tmem_get_label_singular() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>

						<h4><?php printf( __( 'Add %s Types', 'mobile-events-manager' ), tmem_get_label_singular() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></h4>
						<p><?php printf( __( 'If the %1$s type does not exist for the %1$s you are creating, click the <em>Add New</em> link next to the %2$s Type dropdown, enter the %2$s Type name in the text box that is revealed and click <em>Add</em>.<br />To manage all %2$s Types, go to <a href="%4$s">TMEM %3$s &rarr; %2$s Types</a>.', 'mobile-events-manager' ), tmem_get_label_singular( true ), tmem_get_label_singular(), tmem_get_label_plural(), admin_url( 'edit-tags.php?taxonomy=event-types&post_type=tmem-event' ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
					</div>
				</div>
			</div>

			<div class="changelog">
				<h3><?php esc_html_e( 'Setup Templates for Automation', 'mobile-events-manager' ); ?></h3>
				<div class="feature-section">
					<div class="feature-section-media">
						<img src="<?php echo esc_url( TMEM_PLUGIN_URL ) . '/assets/images/screenshots/mem_edit_emails.jpg'; ?>"/>
					</div>
					<div class="feature-section-content">
						<h4><?php esc_html_e( 'Email Templates', 'mobile-events-manager' ); ?></h4>
						<p><?php printf( __( 'Email templates can be configured to be sent automatically during an %1$s status change. Supporting our vast collection of <a href="%2$s" target="_blank">content tags</a> each email can be completley customised and tailored to the %1$s and client details.', 'mobile-events-manager' ), tmem_get_label_singular( true ), 'https://www.mobileeventsmanager.co.uk/docs/content-tags/' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
						<p><?php esc_html_e( 'With email tracking enabled, you can even be sure that your client received your email and know when they have read it.', 'mobile-events-manager' ); ?></p>

						<h4><?php esc_html_e( 'Contract Templates', 'mobile-events-manager' ); ?></h4>
						<p><?php printf( __( 'Create contract templates that be assigned to your %s. Clients will be able to review and <strong>Digitally Sign</strong> the contract via the %2$s.', 'mobile-events-manager' ), tmem_get_label_singular(), tmem_get_application_name() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
					</div>
				</div>
			</div>

			<div class="changelog">
				<h3><?php printf( __( 'Create %s Packages &amp; Add-ons', 'mobile-events-manager' ), tmem_get_label_singular() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></h3>
				<div class="feature-section">
					<div class="feature-section-media">
						<img src="<?php echo esc_url( TMEM_PLUGIN_URL ) . '/assets/images/screenshots/mem_create_addon.jpg'; ?>"/>
					</div>
					<div class="feature-section-content">
						<h4><?php printf( __( '%1$s Packages', 'mobile-events-manager' ), tmem_get_label_singular() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></h4>
						<p><?php printf( __( 'Packages are a pre-defined collection of add-ons that you can offer to your clients for their %s. Define a price for the package and upon selection, the %1$s %2$s and %3$s will be automatically and dynamically re-calculated. Add-ons included within the package, will no longer be available for selection within the add-ons list for this %2$s.', 'mobile-events-manager' ), tmem_get_label_singular( true ), tmem_get_balance_label(), tmem_get_deposit_label() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>

						<h4><?php esc_html_e( 'Add-ons', 'mobile-events-manager' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></h4>
						<p><?php printf( __( 'Add-ons are additional equipment items that can be selected for an %1$s. Each add-on is assigned an individual price and when selected the %1$s %2$s and %3$s are automatically and dynamically re-calculated.', 'mobile-events-manager' ), tmem_get_label_singular( true ), tmem_get_balance_label(), tmem_get_deposit_label() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
						<p><?php printf( __( 'Once you have enabled %1$s Packages &amp; Add-ons within the <a href="%3$s">TMEM %2$s Settings page</a>, manage them within <a href="%4$s">TMEM %2$s &rarr; Equipment Packages</a> and <a href="%5$s">TMEM %2$s &rarr; Equipment Add-ons</a>.', 'mobile-events-manager' ), tmem_get_label_singular( true ), tmem_get_label_plural(), admin_url( 'admin.php?page=tmem-settings&tab=events' ), admin_url( 'edit.php?post_type=tmem-package' ), admin_url( 'edit.php?post_type=tmem-addon' ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
					</div>
				</div>
			</div>

			<div class="changelog">
				<h3><?php esc_html_e( 'Even More Features', 'mobile-events-manager' ); ?></h3>
				<div class="feature-section two-col">
					<div class="col">
						<h4><?php esc_html_e( 'Integrated Client Portal', 'mobile-events-manager' ); ?></h4>
						<p><?php printf( __( 'Known as the <em>Client Portal</em> by default, a password protected portal is available to your clients where they can review their %1$s, view and accept your quote, digitally sign their contract, and manage their %1$s playlist. All %2$s pages use a template system and are fully customisable.', 'mobile-events-manager' ), tmem_get_label_singular( true ), tmem_get_application_name() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
					</div>
					<div class="col">
						<h4><?php esc_html_e( 'Digitally Sign Contracts', 'mobile-events-manager' ); ?></h4>
						<p><?php printf( __( 'Via the %1$s, clients are able to review and digitally sign their %2$s contract. Signing requires confirmation of their name and password for verification to maintain security.', 'mobile-events-manager' ), tmem_get_application_name(), tmem_get_label_singular() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
					</div>
				</div>
				<div class="feature-section two-col">
					<div class="col">
						<h4><?php esc_html_e( 'Transaction Logging', 'mobile-events-manager' ); ?></h4>
						<p><?php printf( __( 'Log all payments your business receives and all expenses you have with the Mobile Events Manager Transactions system. Instantly know how profitable your %s are as well as how much money your company has made over differing periods of time.', 'mobile-events-manager' ), tmem_get_label_plural() ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
					</div>
					<div class="col">
						<h4><?php esc_html_e( 'Multi Employee Aware', 'mobile-events-manager' ); ?></h4>
						<p><?php printf( __( 'Mobile Events Manager supports as many employees as you need at no additional cost. Easily create new employees, set permissions for them to ensure they only have access to what they need, and then assign as many employees to an %s as you need.', 'mobile-events-manager' ), tmem_get_label_singular( true ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
					</div>
				</div>
			</div>

			<div class="changelog">
				<h3><?php esc_html_e( 'Need Help?', 'mobile-events-manager' ); ?></h3>
				<div class="feature-section two-col">
					<div class="col">
						<h4><?php esc_html_e( 'Excellent Support', 'mobile-events-manager' ); ?></h4>
						<p><?php esc_html_e( 'We pride ourselves on our level of support and excellent response times. If you are experiencing an issue, submit a support ticket and we will respond quickly.', 'mobile-events-manager' ); ?></p>
					</div>
					<div class="col">
						<h4><?php esc_html_e( 'Join our Facebook User Group', 'mobile-events-manager' ); ?></h4>
						<p>Our <a href="https://www.facebook.com/groups/mobileeventsmanager/" target="_blank">TMEM Facebook User Group</a> is a great way to exchange knowledge with other users and gain tips for use.</p>
					</div>
				</div>
			</div>

			<div class="changelog">
				<h3><?php esc_html_e( 'Extensions', 'mobile-events-manager' ); ?></h3>
				<div class="feature-section two-col">
					<div class="col">
						<h4><?php esc_html_e( 'A Growing List of Add-ons', 'mobile-events-manager' ); ?></h4>
						<p><?php esc_html_e( 'Add-on plugins are available that greatly extend the default functionality of Mobile Events Manager. There are extensions to further automate Mobile Events Manager, payment processing and calendar syncronisation.', 'mobile-events-manager' ); ?></p>
						<p><?php esc_html_e( 'Further more, if you have migrated from any other plugin, we will try our best to make available the same addons. Just ask!', 'mobile-events-manager' ); ?></p>
					</div>
					<div class="col">
						<h4><?php esc_html_e( 'Visit the Add-ons Store', 'mobile-events-manager' ); ?></h4>
						<p><?php printf( __( '<a href="%s" target="_blank">The Add-ons store</a> currently has an Add-On which allows you to take secure payments via Stripe. We are currently working on more so be sure to sign up on our website to find out about the releases.', 'mobile-events-manager' ), 'https://www.mobileeventsmanager.co.uk/add-ons' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?></p>
						<p><?php esc_html_e( 'There are also another two add-ons available as well. We have an Enquiry Forms plugin which piggybacks from CF7 and also a real-time Google Calendar Integration plugin as well. The Enquiry Forms extension is available in the add-ons store and the Google Calendar Integration one is available from the WordPress Plugin repository.', 'mobile-events-manager' ); ?></p>
					</div>
				</div>
			</div>
		</div>
		<?php
	} // getting_started_screen

	/**
	 * Parse the TMEM readme.txt file
	 *
	 * @since 2.0.3
	 * @return string $readme HTML formatted readme file
	 */
	public function parse_readme() {
		$file = file_exists( TMEM_PLUGIN_DIR . '/readme.txt' ) ? TMEM_PLUGIN_DIR . '/readme.txt' : null;

		if ( ! $file ) {
			$readme = '<p>' . __( 'No valid changelog was found.', 'mobile-events-manager' ) . '</p>';
		} else {
			$readme = file_get_contents( $file );
			$readme = nl2br( esc_html( $readme ) );
			$readme = explode( '== Changelog ==', $readme );
			$readme = end( $readme );

			$readme = preg_replace( '/`(.*?)`/', '<code>\\1</code>', $readme );
			$readme = preg_replace( '/[\040]\*\*(.*?)\*\*/', ' <strong>\\1</strong>', $readme );
			$readme = preg_replace( '/[\040]\*(.*?)\*/', ' <em>\\1</em>', $readme );
			$readme = preg_replace( '/= (.*?) =/', '<h4>\\1</h4>', $readme );
			$readme = preg_replace( '/\[(.*?)\]\((.*?)\)/', '<a href="\\2">\\1</a>', $readme );
		}

		return $readme;
	} // parse_readme

	/**
	 * Sends user to the Welcome page on first activation of TMEM as well as each
	 * time TMEM is upgraded to a new version
	 *
	 * @access public
	 * @since 1.3
	 * @return void
	 */
	public function welcome() {
		// Bail if no activation redirect
		if ( ! get_transient( '_tmem_activation_redirect' ) ) {
			return;
		}

		// Delete the redirect transient
		delete_transient( '_tmem_activation_redirect' );

		// Bail if activating from network, or bulk
		if ( is_network_admin() || isset( $_GET['activate-multi'] ) ) {
			return;
		}

		$upgrade = get_option( 'tmem_version_upgraded_from' );

		// if ( ! $upgrade ) { // First time install
			wp_safe_redirect( admin_url( 'index.php?page=tmem-getting-started' ) );
			exit;
		// } else { // Update
		// wp_safe_redirect( admin_url( 'index.php?page=tmem-about' ) );
		// exit;
		// }
	} // welcome
}
new TMEM_Welcome();
