<?php


/**
 * Travel Class
 *
 * @package TMEM
 * @subpackage Classes/Travel
 * @copyright Copyright (c) 2016, Mike Howard
 * @license http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since 1.4
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * TMEM_Travel Class
 *
 * @since 1.4
 */
class TMEM_Travel {

	/**
	 * The start address
	 *
	 * @since 1.4
	 * @var array $start_address the start address.
	 */
	public $start_address;

	/**
	 * The destination address
	 *
	 * @since 1.4
	 * @var array $destination_address destination address
	 */
	public $destination_address;

	/**
	 * The distance of travel
	 *
	 * @since 1.4
	 * @var int $distance distance of travel.
	 */
	public $distance;

	/**
	 * The cost of travel
	 *
	 * @since 1.4
	 * @var int $cost the cost of travel
	 */
	public $cost = '0.00';

	/**
	 * Whether or not to add travel costs to an event
	 *
	 * @since 1.4
	 * @var array $add_travel_cost add travel costs.
	 */
	public $add_travel_cost = false;

	/**
	 * The travel data.
	 *
	 * @since 1.4
	 * @var array $data the travel data.
	 */
	public $data;

	/**
	 * The travel mode.
	 *
	 * @since 1.4
	 * @var array $mode travel mode. Default driving.
	 */
	public $mode = 'driving';

	/**
	 * Units.
	 *
	 * @since 1.4
	 * @var array $units The unit of measurement.
	 */
	public $units;

	/**
	 * API key
	 *
	 * @access private
	 * @since 1.4
	 * @var array $api_key API Key for Google Maps
	 */
	private $api_key;

	/**
	 * Maps.
	 *
	 * @access private
	 * @since 1.4
	 * @var array $maps Google Maps
	 */
	private $maps = 'https://maps.googleapis.com/maps/api/distancematrix/json';

	/**
	 * Directions.
	 *
	 * @access private
	 * @since 1.4
	 * @var array $directions The Directions.
	 */
	private $directions = 'https://maps.google.com/maps';

	/**
	 * The query.
	 *
	 * @access private
	 * @since 1.4
	 * @var array $query The query.
	 */
	private $query = false;

	/**
	 * Constructor
	 *
	 * @since 1.4
	 * @param array $args constructor argument.
	 */
	public function __construct( $args = array() ) {
		$this->init();
	} // __construct

	/**
	 * Init.
	 *
	 * @access private
	 * @since 1.4
	 */
	private function init() {
		if ( ! isset( $this->start_address ) ) {
			$this->start_address = tmem_get_option( 'travel_primary' );
		}

		$this->units           = tmem_get_option( 'travel_units' );
		$this->add_travel_cost = tmem_get_option( 'travel_add_cost' );
	} // init

	/**
	 * Set a property
	 *
	 * @since 1.4
	 * @param int $key call to get private property.
	 * @param int $value value of something.
	 */
	public function __set( $key, $value ) {
		$this->$key = $value;
	} // __set

	/**
	 * Magic __get function to dispatch a call to retrieve a private property
	 *
	 * @since 1.4
	 * @param int $key call to get private property.
	 */
	public function __get( $key ) {
		if ( method_exists( $this, 'get_' . $key ) ) {
			return call_user_func( array( $this, 'get_' . $key ) );
		} else {
			/* translators: %s: name/address of property */
			return new WP_Error( 'tmem-travel-invalid-property', sprintf( esc_html__( "Can't get property %s", 'mobile-events-manager' ), $key ) );
		}
	} // __get

	/**
	 * Prepare query data.
	 *
	 * @access private
	 * @since 1.4
	 */
	private function prepare_query() {
		if ( ! isset( $this->destination_address ) ) {
			return false;
		}

		if ( is_array( $this->start_address ) ) {
			$this->start_address = implode( ',', $this->start_address );
		}

		if ( is_array( $this->destination_address ) ) {
			$this->destination_address = implode( ',', $this->destination_address );
		}

		$query_args = array(
			'units'        => $this->units,
			'mode'         => $this->mode,
			'origins'      => str_replace( '%2C', ',', rawurlencode( $this->start_address ) ),
			'destinations' => str_replace( '%2C', ',', rawurlencode( $this->destination_address ) ),
		);

		/*
		 * Allow filtering of the query args.
		 *
		 * @since	1.4
		 */
		$query_args = apply_filters( 'tmem_travel_query_args', $query_args, $this );

		$this->query = add_query_arg( $query_args, $this->maps );
	} // prepare_query

	/**
	 * Process the query.
	 *
	 * @since 1.4
	 */
	public function process_query() {
		$this->prepare_query();

		if ( ! isset( $this->query ) ) {
			return false;
		}

		$response = wp_remote_get( $this->query );

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$travel_data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( empty( $travel_data ) || 'OK' !== $travel_data->status ) {
			return false;
		}

		if ( empty( $travel_data->rows ) ) {
			return false;
		}

		if ( empty( $travel_data->origin_addresses[0] ) || empty( $travel_data->destination_addresses[0] ) ) {
			return false;
		}

		if ( empty( $travel_data->rows[0]->elements[0]->distance->value ) || empty( $travel_data->rows[0]->elements[0]->duration->value ) ) {
			return false;
		}

		$this->distance = str_replace(
			array( 'km', 'mi' ),
			array( '', '' ),
			$travel_data->rows[0]->elements[0]->distance->text
		);

		$travel = array(
			'origin'      => $travel_data->origin_addresses[0],
			'destination' => $travel_data->destination_addresses[0],
			'duration'    => $travel_data->rows[0]->elements[0]->duration->value,
			'distance'    => $this->distance,
		);

		$travel = apply_filters( 'tmem_travel_data', $travel, $travel_data );

		return $travel;

	} // process_query

	/**
	 * Retrieve the travel data.
	 *
	 * @since 1.4
	 */
	public function get_travel_data() {
		if ( ! isset( $this->destination_address ) ) {
			return false;
		}

		$this->data = $this->process_query();

		return apply_filters( 'tmem_get_travel_data', $this->data, $this );
	} // get_travel_data

	/**
	 * Retrieve the cost of the trip.
	 *
	 * @since 1.4
	 */
	function get_cost() {
		if ( ! $this->add_travel_cost ) {
			return 0;
		}

		$min       = tmem_get_option( 'travel_min_distance' );
		$unit_cost = tmem_get_option( 'cost_per_unit' );
		$round     = tmem_get_option( 'travel_cost_round' );

		if ( intval( $this->distance ) >= $min ) {
			$this->cost = $this->distance * $unit_cost;

			if ( $round ) {
				$nearest = tmem_get_option( 'travel_round_to' );

				if ( intval( $this->cost ) === $this->cost && ! is_float( intval( $this->cost ) / $nearest ) ) {
					$this->cost = intval( $this->cost );
				} else {
					if ( 'up' === $round ) {
						$this->cost = round( ( $this->cost + $nearest / 2 ) / $nearest ) * $nearest;
					} else {
						$this->cost = floor( ( $this->cost + $nearest / 2 ) / $nearest ) * $nearest;
					}
				}
			}
		}

		return apply_filters( 'tmem_travel_cost', $this->cost, $this );
	} // get_cost

	/**
	 * Retrieve a venue address.
	 *
	 * @since 1.4
	 * @param int $id Post ID of either an event or venue.
	 * @return arr|false $address Array of address fields, or false.
	 */
	public function get_venue_address( $id ) {
		$post_type = get_post_type( $id );

		if ( 'tmem-event' !== $post_type && 'tmem-venue' !== $post_type ) {
			return false;
		}

		$venue_address = tmem_get_event_venue_meta( $id, 'address' );

		$address = ! empty( $venue_address ) ? $venue_address : false;
		$address = apply_filters( 'tmem_get_venue_address', $address, $id );

		return $address;
	} // get_venue_address

	/**
	 * Retrieve an employees address.
	 *
	 * @since 1.4
	 * @param int $employee_id User ID of an employee.
	 * @return arr|false $address Array of address fields, or false.
	 */
	public function get_employee_address( $employee_id ) {

		if ( ! tmem_is_employee( $employee_id ) ) {
			return false;
		}

		$employee_address = tmem_get_employee_address( $employee_id );

		if ( is_array( $employee_address ) ) {
			$employee_address = implode( ',', array_filter( $employee_address ) );
		}

		// To filter this use apply_filters( 'tmem_get_employee_address', $address, $user_id );.
		$address = ! empty( $employee_address ) ? $employee_address : false;

		return $address;
	} // get_employee_address

	/**
	 * Retrieve a clients address.
	 *
	 * @since 1.4
	 * @param int $client_id User ID of an employee.
	 * @return arr|false $address Array of address fields, or false.
	 */
	public function get_client_address( $client_id ) {
		$client_address = tmem_get_client_address( $client_id );

		if ( is_array( $client_address ) ) {
			$client_address = implode( ',', array_filter( $client_address ) );
		}

		// To filter this use apply_filters( 'tmem_get_client_address', $address, $client_id );.
		$address = ! empty( $client_address ) ? $client_address : false;

		return $address;
	} // get_client_address

	/**
	 * Set the destination address.
	 *
	 * @since 1.4
	 * @param int|arr|obj $dest An event ID or object, or a venue post ID or an address array.
	 * @return void
	 */
	public function set_destination( $dest ) {
		if ( is_array( $dest ) ) { // Address array is passed.
			$dest                      = implode( ',', array_filter( $dest ) );
			$this->destination_address = $dest;
		} elseif ( is_object( $dest ) ) { // Event object is passed.
			$tmem_event                = new TMEM_Event( $event );
			$this->destination_address = $this->get_venue_address( $tmem_event->get_venue_id() );
		} elseif ( is_numeric( $dest ) ) { // Event or Venue ID is passed.
			$this->destination_address = $this->get_venue_address( $dest );
		}
	} // set_destination

	/**
	 * Retrieve the URL for directions.
	 *
	 * @since 1.4
	 */
	function get_directions_url() {
		if ( ! isset( $this->destination_address ) ) {
			return false;
		}

		if ( is_array( $this->start_address ) ) {
			$this->start_address = implode( ',', $this->start_address );
		}

		if ( is_array( $this->destination_address ) ) {
			$this->destination_address = implode( ',', $this->destination_address );
		}

		$url = add_query_arg(
			array(
				'dirflg' => 'd',
				'saddr'  => rawurlencode( $this->start_address ),
				'daddr'  => rawurlencode( $this->destination_address ),
			),
			$this->directions
		);

		return apply_filters( 'tmem_directions_url', $url, $this );

	} // get_directions_url

} // class TMEM_Travel
